@@ -287,7 +287,7 @@ typedef Open_tables_backup SPIDER_Open_tables_backup;
 #define SPIDER_LINK_MON_DRAW_FEW_MON         1
 #define SPIDER_LINK_MON_DRAW                 2
 
-#define SPIDER_TMP_SHARE_CHAR_PTR_COUNT     21
+#define SPIDER_TMP_SHARE_CHAR_PTR_COUNT     23
 #define SPIDER_TMP_SHARE_UINT_COUNT         SPIDER_TMP_SHARE_CHAR_PTR_COUNT
 #define SPIDER_TMP_SHARE_LONG_COUNT         20
 #define SPIDER_TMP_SHARE_LONGLONG_COUNT      3
@@ -383,6 +383,8 @@ typedef struct st_spider_alter_table
   char               **tmp_tgt_default_files;
   char               **tmp_tgt_default_groups;
   char               **tmp_tgt_dsns;
+  char               **tmp_tgt_filedsns;
+  char               **tmp_tgt_drivers;
   char               **tmp_static_link_ids;
   long               *tmp_tgt_ports;
   long               *tmp_tgt_ssl_vscs;
@@ -405,6 +407,8 @@ typedef struct st_spider_alter_table
   uint               *tmp_tgt_default_files_lengths;
   uint               *tmp_tgt_default_groups_lengths;
   uint               *tmp_tgt_dsns_lengths;
+  uint               *tmp_tgt_filedsns_lengths;
+  uint               *tmp_tgt_drivers_lengths;
   uint               *tmp_static_link_ids_lengths;
 
   uint               tmp_server_names_charlen;
@@ -423,6 +427,8 @@ typedef struct st_spider_alter_table
   uint               tmp_tgt_default_files_charlen;
   uint               tmp_tgt_default_groups_charlen;
   uint               tmp_tgt_dsns_charlen;
+  uint               tmp_tgt_filedsns_charlen;
+  uint               tmp_tgt_drivers_charlen;
   uint               tmp_static_link_ids_charlen;
 
   uint               tmp_server_names_length;
@@ -441,6 +447,8 @@ typedef struct st_spider_alter_table
   uint               tmp_tgt_default_files_length;
   uint               tmp_tgt_default_groups_length;
   uint               tmp_tgt_dsns_length;
+  uint               tmp_tgt_filedsns_length;
+  uint               tmp_tgt_drivers_length;
   uint               tmp_static_link_ids_length;
   uint               tmp_tgt_ports_length;
   uint               tmp_tgt_ssl_vscs_length;
@@ -461,11 +469,6 @@ typedef struct st_spider_conn
 #endif
   int                link_idx;
   spider_db_conn     *db_conn;
-#if defined(HS_HAS_SQLCOM) && defined(HAVE_HANDLERSOCKET)
-  query_id_t         hsc_query_id;
-  ulonglong          hs_pre_age;
-  ulonglong          hs_age;
-#endif
   uint               opened_handlers;
   ulonglong          conn_id;
   ulonglong          connection_id;
@@ -526,12 +529,10 @@ typedef struct st_spider_conn
   char               *tgt_default_file;
   char               *tgt_default_group;
   char               *tgt_dsn;
+  char               *tgt_filedsn;
+  char               *tgt_driver;
   long               tgt_port;
   long               tgt_ssl_vsc;
-#if defined(HS_HAS_SQLCOM) && defined(HAVE_HANDLERSOCKET)
-  char               *hs_sock;
-  long               hs_port;
-#endif
 
   uint               tgt_host_length;
   uint               tgt_username_length;
@@ -547,9 +548,8 @@ typedef struct st_spider_conn
   uint               tgt_default_file_length;
   uint               tgt_default_group_length;
   uint               tgt_dsn_length;
-#if defined(HS_HAS_SQLCOM) && defined(HAVE_HANDLERSOCKET)
-  uint               hs_sock_length;
-#endif
+  uint               tgt_filedsn_length;
+  uint               tgt_driver_length;
   uint               dbton_id;
 
 #ifndef WITHOUT_SPIDER_BG_SEARCH
@@ -736,11 +736,6 @@ typedef struct st_spider_wide_handler
   uchar              *rnd_write_bitmap;
   SPIDER_CONDITION   *condition;
   void               *owner;
-#if defined(HS_HAS_SQLCOM) && defined(HAVE_HANDLERSOCKET)
-#ifdef HANDLER_HAS_DIRECT_UPDATE_ROWS
-  uint32             *hs_pushed_ret_fields;
-#endif
-#endif
 #ifdef WITH_PARTITION_STORAGE_ENGINE
   SPIDER_PARTITION_HANDLER *partition_handler;
 #endif
@@ -751,11 +746,6 @@ typedef struct st_spider_wide_handler
   TABLE_SHARE        *top_share;
   enum thr_lock_type lock_type;
   uchar              lock_table_type;
-#if defined(HS_HAS_SQLCOM) && defined(HAVE_HANDLERSOCKET)
-#ifdef HANDLER_HAS_DIRECT_UPDATE_ROWS
-  uint32             hs_pushed_strref_num;
-#endif
-#endif
   int                lock_mode;
   int                external_lock_type;
   int                cond_check_error;
@@ -768,13 +758,6 @@ typedef struct st_spider_wide_handler
 #endif
 #ifdef HA_CAN_BULK_ACCESS
   ulonglong          external_lock_cnt;
-#endif
-#if defined(HS_HAS_SQLCOM) && defined(HAVE_HANDLERSOCKET)
-#ifdef HANDLER_HAS_DIRECT_UPDATE_ROWS
-  size_t             hs_pushed_ret_fields_num;
-  size_t             hs_pushed_ret_fields_size;
-  size_t             hs_pushed_lcl_fields_num;
-#endif
 #endif
   bool               between_flg;
   bool               idx_bitmap_is_set;
@@ -784,7 +767,6 @@ typedef struct st_spider_wide_handler
   bool               semi_trx_chk;
   bool               low_priority;
   bool               high_priority;
-  bool               insert_delayed;
   bool               consistent_snapshot;
   bool               quick_mode;
   bool               keyread;
@@ -793,12 +775,6 @@ typedef struct st_spider_wide_handler
   bool               write_can_replace;
   bool               insert_with_update;
   bool               cond_check;
-#if defined(HS_HAS_SQLCOM) && defined(HAVE_HANDLERSOCKET)
-#ifdef HANDLER_HAS_DIRECT_UPDATE_ROWS
-  bool               hs_increment;
-  bool               hs_decrement;
-#endif
-#endif
   bool               semi_table_lock;
 } SPIDER_WIDE_HANDLER;
 
@@ -834,30 +810,6 @@ typedef struct st_spider_transaction
   const char         *trx_another_conn_hash_func_name;
   const char         *trx_another_conn_hash_file_name;
   ulong              trx_another_conn_hash_line_no;
-#if defined(HS_HAS_SQLCOM) && defined(HAVE_HANDLERSOCKET)
-  HASH               trx_hs_r_conn_hash;
-  uint               trx_hs_r_conn_hash_id;
-  const char         *trx_hs_r_conn_hash_func_name;
-  const char         *trx_hs_r_conn_hash_file_name;
-  ulong              trx_hs_r_conn_hash_line_no;
-  HASH               trx_hs_w_conn_hash;
-  uint               trx_hs_w_conn_hash_id;
-  const char         *trx_hs_w_conn_hash_func_name;
-  const char         *trx_hs_w_conn_hash_file_name;
-  ulong              trx_hs_w_conn_hash_line_no;
-#endif
-#if defined(HS_HAS_SQLCOM) && defined(HAVE_HANDLERSOCKET)
-  HASH               trx_direct_hs_r_conn_hash;
-  uint               trx_direct_hs_r_conn_hash_id;
-  const char         *trx_direct_hs_r_conn_hash_func_name;
-  const char         *trx_direct_hs_r_conn_hash_file_name;
-  ulong              trx_direct_hs_r_conn_hash_line_no;
-  HASH               trx_direct_hs_w_conn_hash;
-  uint               trx_direct_hs_w_conn_hash_id;
-  const char         *trx_direct_hs_w_conn_hash_func_name;
-  const char         *trx_direct_hs_w_conn_hash_file_name;
-  ulong              trx_direct_hs_w_conn_hash_line_no;
-#endif
   HASH               trx_alter_table_hash;
   uint               trx_alter_table_hash_id;
   const char         *trx_alter_table_hash_func_name;
@@ -873,10 +825,6 @@ typedef struct st_spider_transaction
   SPIDER_CONN        *join_trx_top;
   ulonglong          spider_thread_id;
   ulonglong          trx_conn_adjustment;
-#if defined(HS_HAS_SQLCOM) && defined(HAVE_HANDLERSOCKET)
-  ulonglong          trx_hs_r_conn_adjustment;
-  ulonglong          trx_hs_w_conn_adjustment;
-#endif
   uint               locked_connections;
 
   ulonglong          direct_update_count;
@@ -884,9 +832,6 @@ typedef struct st_spider_transaction
   ulonglong          direct_order_limit_count;
   ulonglong          direct_aggregate_count;
   ulonglong          parallel_search_count;
-#if defined(HS_HAS_SQLCOM) && defined(HAVE_HANDLERSOCKET)
-  ulonglong          hs_result_free_count;
-#endif
 
 #ifdef HA_CAN_BULK_ACCESS
   SPIDER_CONN        *bulk_access_conn_first;
@@ -1069,7 +1014,6 @@ typedef struct st_spider_share
   int                selupd_lock_mode;
   int                query_cache;
   int                query_cache_sync;
-  int                internal_delayed;
   int                bulk_size;
   int                bulk_update_mode;
   int                bulk_update_size;
@@ -1103,9 +1047,6 @@ typedef struct st_spider_share
   int                error_read_mode;
   int                error_write_mode;
   int                active_link_count;
-#if defined(HS_HAS_SQLCOM) && defined(HAVE_HANDLERSOCKET)
-  longlong           hs_result_free_size;
-#endif
 #ifdef HA_CAN_BULK_ACCESS
   int                bulk_access_free;
 #endif
@@ -1124,10 +1065,6 @@ typedef struct st_spider_share
 
 #ifdef SPIDER_HAS_HASH_VALUE_TYPE
   my_hash_value_type *conn_keys_hash_value;
-#if defined(HS_HAS_SQLCOM) && defined(HAVE_HANDLERSOCKET)
-  my_hash_value_type *hs_read_conn_keys_hash_value;
-  my_hash_value_type *hs_write_conn_keys_hash_value;
-#endif
 #endif
   char               **server_names;
   char               **tgt_table_names;
@@ -1145,16 +1082,12 @@ typedef struct st_spider_share
   char               **tgt_default_files;
   char               **tgt_default_groups;
   char               **tgt_dsns;
+  char               **tgt_filedsns;
+  char               **tgt_drivers;
   char               **static_link_ids;
   char               **tgt_pk_names;
   char               **tgt_sequence_names;
   char               **conn_keys;
-#if defined(HS_HAS_SQLCOM) && defined(HAVE_HANDLERSOCKET)
-  char               **hs_read_socks;
-  char               **hs_write_socks;
-  char               **hs_read_conn_keys;
-  char               **hs_write_conn_keys;
-#endif
   long               *tgt_ports;
   long               *tgt_ssl_vscs;
   long               *link_statuses;
@@ -1170,13 +1103,6 @@ typedef struct st_spider_share
 #endif
   longlong           *monitoring_limit;
   longlong           *monitoring_sid;
-#if defined(HS_HAS_SQLCOM) && defined(HAVE_HANDLERSOCKET)
-  long               *use_hs_reads;
-  long               *use_hs_writes;
-  long               *hs_read_ports;
-  long               *hs_write_ports;
-  long               *hs_write_to_reads;
-#endif
   long               *use_handlers;
   long               *connect_timeouts;
   long               *net_read_timeouts;
@@ -1201,20 +1127,13 @@ typedef struct st_spider_share
   uint               *tgt_default_files_lengths;
   uint               *tgt_default_groups_lengths;
   uint               *tgt_dsns_lengths;
+  uint               *tgt_filedsns_lengths;
+  uint               *tgt_drivers_lengths;
   uint               *static_link_ids_lengths;
   uint               *tgt_pk_names_lengths;
   uint               *tgt_sequence_names_lengths;
   uint               *conn_keys_lengths;
-#if defined(HS_HAS_SQLCOM) && defined(HAVE_HANDLERSOCKET)
-  uint               *hs_read_socks_lengths;
-  uint               *hs_write_socks_lengths;
-  uint               *hs_read_conn_keys_lengths;
-  uint               *hs_write_conn_keys_lengths;
-#endif
   uint               *sql_dbton_ids;
-#if defined(HS_HAS_SQLCOM) && defined(HAVE_HANDLERSOCKET)
-  uint               *hs_dbton_ids;
-#endif
 
   uint               server_names_charlen;
   uint               tgt_table_names_charlen;
@@ -1232,16 +1151,12 @@ typedef struct st_spider_share
   uint               tgt_default_files_charlen;
   uint               tgt_default_groups_charlen;
   uint               tgt_dsns_charlen;
+  uint               tgt_filedsns_charlen;
+  uint               tgt_drivers_charlen;
   uint               static_link_ids_charlen;
   uint               tgt_pk_names_charlen;
   uint               tgt_sequence_names_charlen;
   uint               conn_keys_charlen;
-#if defined(HS_HAS_SQLCOM) && defined(HAVE_HANDLERSOCKET)
-  uint               hs_read_socks_charlen;
-  uint               hs_write_socks_charlen;
-  uint               hs_read_conn_keys_charlen;
-  uint               hs_write_conn_keys_charlen;
-#endif
 
   uint               server_names_length;
   uint               tgt_table_names_length;
@@ -1259,16 +1174,12 @@ typedef struct st_spider_share
   uint               tgt_default_files_length;
   uint               tgt_default_groups_length;
   uint               tgt_dsns_length;
+  uint               tgt_filedsns_length;
+  uint               tgt_drivers_length;
   uint               static_link_ids_length;
   uint               tgt_pk_names_length;
   uint               tgt_sequence_names_length;
   uint               conn_keys_length;
-#if defined(HS_HAS_SQLCOM) && defined(HAVE_HANDLERSOCKET)
-  uint               hs_read_socks_length;
-  uint               hs_write_socks_length;
-  uint               hs_read_conn_keys_length;
-  uint               hs_write_conn_keys_length;
-#endif
   uint               tgt_ports_length;
   uint               tgt_ssl_vscs_length;
   uint               link_statuses_length;
@@ -1284,13 +1195,6 @@ typedef struct st_spider_share
 #endif
   uint               monitoring_limit_length;
   uint               monitoring_sid_length;
-#if defined(HS_HAS_SQLCOM) && defined(HAVE_HANDLERSOCKET)
-  uint               use_hs_reads_length;
-  uint               use_hs_writes_length;
-  uint               hs_read_ports_length;
-  uint               hs_write_ports_length;
-  uint               hs_write_to_reads_length;
-#endif
   uint               use_handlers_length;
   uint               connect_timeouts_length;
   uint               net_read_timeouts_length;
@@ -1308,11 +1212,6 @@ typedef struct st_spider_share
   uint               use_sql_dbton_count;
   uint               use_sql_dbton_ids[SPIDER_DBTON_SIZE];
   uint               sql_dbton_id_to_seq[SPIDER_DBTON_SIZE];
-#if defined(HS_HAS_SQLCOM) && defined(HAVE_HANDLERSOCKET)
-  uint               use_hs_dbton_count;
-  uint               use_hs_dbton_ids[SPIDER_DBTON_SIZE];
-  uint               hs_dbton_id_to_seq[SPIDER_DBTON_SIZE];
-#endif
 
   SPIDER_ALTER_TABLE alter_table;
   SPIDER_WIDE_SHARE  *wide_share;
@@ -1371,9 +1270,6 @@ typedef struct st_spider_direct_sql
   int                  net_write_timeout;
   longlong             bulk_insert_rows;
   int                  connection_channel;
-#if defined(HS_HAS_SQLCOM) && defined(HAVE_HANDLERSOCKET)
-  int                  access_mode;
-#endif
 #if MYSQL_VERSION_ID < 50500
 #else
   int                  use_real_table;
@@ -1395,6 +1291,8 @@ typedef struct st_spider_direct_sql
   char                 *tgt_default_file;
   char                 *tgt_default_group;
   char                 *tgt_dsn;
+  char                 *tgt_filedsn;
+  char                 *tgt_driver;
   char                 *conn_key;
   long                 tgt_port;
   long                 tgt_ssl_vsc;
@@ -1414,6 +1312,8 @@ typedef struct st_spider_direct_sql
   uint                 tgt_default_file_length;
   uint                 tgt_default_group_length;
   uint                 tgt_dsn_length;
+  uint                 tgt_filedsn_length;
+  uint                 tgt_driver_length;
   uint                 conn_key_length;
   uint                 dbton_id;
 #ifdef SPIDER_HAS_HASH_VALUE_TYPE
